@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2017 Brazil
+  Copyright(C) 2009-2018 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -288,6 +288,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
     ctx->impl->match_escalation_threshold =
       grn_get_default_match_escalation_threshold();
   }
+  ctx->impl->force_match_escalation = GRN_FALSE;
 
   ctx->impl->finalizer = NULL;
 
@@ -905,6 +906,27 @@ grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, long long int threshold)
   return GRN_SUCCESS;
 }
 
+grn_bool
+grn_ctx_get_force_match_escalation(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->force_match_escalation;
+  } else {
+    return GRN_FALSE;
+  }
+}
+
+grn_rc
+grn_ctx_set_force_match_escalation(grn_ctx *ctx, grn_bool force)
+{
+  if (ctx->impl) {
+    ctx->impl->force_match_escalation = force;
+    return GRN_SUCCESS;
+  } else {
+    return GRN_INVALID_ARGUMENT;
+  }
+}
+
 grn_content_type
 grn_get_ctype(grn_obj *var)
 {
@@ -1633,7 +1655,8 @@ exception_filter(EXCEPTION_POINTERS *info)
     base_dir = grn_windows_base_dir();
     {
       char *current, *end;
-      const char *pdb_dir = "\\lib\\pdb";
+      /* TODO: Add more directories for plugins and so on. */
+      const char *bin_dir = "\\bin";
       current = search_path + strlen(search_path);
       end = current + sizeof(search_path) - 1;
       for (; *base_dir && current < end; base_dir++, current++) {
@@ -1647,7 +1670,7 @@ exception_filter(EXCEPTION_POINTERS *info)
         current--;
       }
       *current = '\0';
-      grn_strcat(search_path, sizeof(search_path), pdb_dir);
+      grn_strcat(search_path, sizeof(search_path), bin_dir);
     }
 
     SymSetSearchPath(process, search_path);
@@ -1680,7 +1703,7 @@ exception_filter(EXCEPTION_POINTERS *info)
 
   previous_address = 0;
   while (GRN_TRUE) {
-    DWORD address;
+    DWORD64 address;
     IMAGEHLP_MODULE64 module;
     char *buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME * sizeof(TCHAR)];
     SYMBOL_INFO *symbol = (SYMBOL_INFO *)buffer;
@@ -1726,11 +1749,10 @@ exception_filter(EXCEPTION_POINTERS *info)
     {
       const char *unknown = "(unknown)";
       GRN_LOG(ctx, GRN_LOG_CRIT,
-              "%s:%d:%d: %p: %.*s(): <%s>: <%s>",
+              "%s:%d:%d: %.*s(): <%s>: <%s>",
               (have_location ? line.FileName : unknown),
               (have_location ? line.LineNumber : -1),
               (have_location ? line_displacement : -1),
-              address,
               (have_symbol_name ? symbol->NameLen : strlen(unknown)),
               (have_symbol_name ? symbol->Name : unknown),
               (have_module_name ? module.ModuleName : unknown),
