@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2009-2018 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -48,6 +48,7 @@ struct _grn_db {
   grn_hash *config;
   grn_tiny_array values;
   grn_critical_section lock;
+  grn_cache *cache;
 };
 
 #define GRN_SERIALIZED_SPEC_INDEX_SPEC   0
@@ -127,6 +128,7 @@ void grn_db_generate_pathname(grn_ctx *ctx,
                               grn_obj *db,
                               grn_id id,
                               char *buffer);
+grn_rc grn_db_clear_dirty(grn_ctx *ctx, grn_obj *db);
 
 grn_rc _grn_table_delete_by_id(grn_ctx *ctx, grn_obj *table, grn_id id,
                                grn_table_delete_optarg *optarg);
@@ -243,6 +245,7 @@ struct _grn_proc {
     struct {
       grn_selector_func *selector;
       grn_operator selector_op;
+      grn_bool is_stable;
     } function;
     struct {
       grn_command_run_func *run;
